<?php

namespace Drupal\migration_notify;

use Drupal\migrate\Plugin\MigrationPluginManager;
use Drupal\migrate\Plugin\MigrationInterface;
use Drupal\migrate\Plugin\RequirementsInterface;
use Drupal\migrate\Exception\RequirementsException;

/**
 * Class NotifierService.
 */
class HelperService {
  /**
   * Drupal\migrate\Plugin\MigrationPluginManager definition.
   *
   * @var \Drupal\migrate\Plugin\MigrationPluginManager
   */
  protected $migrations;

  /**
   * Constructs a new HelperService object.
   */
  public function __construct(MigrationPluginManager $migrations) {
    $this->migrations = $migrations;
  }

  /**
   * Check for migrations that can be stuck.
   *
   * @return bool
   *   Whether there are migrations stuck or not.
   */
  public function checkStuck() {
    $migrations = $this->migrations->createInstances([]);
    $return = FALSE;

    /** @var \Drupal\migrate\Plugin\Migration $migration */
    foreach ($migrations as $id => $migration) {
      $migration = $this->migrations->createInstance($id);
      if ($migration) {
        $status = $migration->getStatus();
        if ($status !== MigrationInterface::STATUS_IDLE) {
          $return = TRUE;
        }
      }
    }

    return $return;
  }

  /**
   * Reset the migrations that can be stuck.
   *
   * @return bool
   *   Whether the migrations were reset or not.
   */
  public function resetMigrations() {
    $migrations = $this->migrations->createInstances([]);
    $errors = [];

    /** @var \Drupal\migrate\Plugin\Migration $migration */
    foreach ($migrations as $id => $migration) {
      if ($migration->getSourcePlugin() instanceof RequirementsInterface) {
        try {
          $migration->getSourcePlugin()->checkRequirements();
          if (!$this->resetMigration($id)) {
            $errors[$id] = TRUE;
          }
        }
        catch (RequirementsException $e) {
          unset($migrations[$id]);
        }
      }
    }

    return empty($errors);
  }

  /**
   * Reset invidivual migration.
   *
   * @param string $name
   *   Name of the migration to reset.
   *
   * @return bool
   *   Whether the migration was reset or not.
   */
  protected function resetMigration($name) {
    $migration = $this->migrations->createInstance($name);
    if ($migration) {
      $status = $migration->getStatus();
      if ($status !== MigrationInterface::STATUS_IDLE) {
        $migration->setStatus(MigrationInterface::STATUS_IDLE);
      }
      return TRUE;
    }

    return FALSE;
  }

}
